#!/usr/bin/python

import random
import re
import textwrap
import signal
import gtk
import gtk.gdk
import pango
import gconf
import gnome.applet
import gnome.ui
import FishTPServer
from Fish import Fish
from OceanConfig import OceanConfig
from PeerManager import PeerManager
from blogfish.blogfish_functions import debug, get_config_value

TheOcean = 0

class Ocean:

	def __init__(self, applet, host=None):
		global TheOcean

		self.client = gconf.client_get_default()
		client = self.client
		client.add_dir("/apps/blogfish", gconf.CLIENT_PRELOAD_RECURSIVE)
		for key in ["URL", "blogURL", "email", "fishName", "fishFile", \
				"fishWords", "backgroundFile", "maxFish", "speed",
				"sexuality", "aggression", "spawningRate", "deathRate"]:
			self.__dict__[key] = get_config_value(key)
			client.notify_add("/apps/blogfish/" + key, self.config_changed)
		self.applet = applet
		self.poss_host = host
		self.fishes = []
		self.texts = []
		self.peer_manager = PeerManager(self)
		self.config_dialog = None
		self.tooltip = gtk.Tooltips()
		TheOcean = self

		self.tank = gtk.gdk.pixbuf_new_from_file(self.backgroundFile)
		self.fish_cursor = gtk.gdk.Cursor(gtk.gdk.HAND2)
		self.turn_timeout = None
	
		targets = [("text/plain", 0, 2), ('STRING', 0, 3), ('TEXT', 0, 4)]
		applet.drag_dest_set(gtk.DEST_DEFAULT_ALL, targets, \
				gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_COPY | \
				gtk.gdk.ACTION_DEFAULT)
		applet.connect("drag_data_received", self._drag_data_received)
		
		applet.connect("button_press_event", self._button_pressed)
		applet.add_events(gtk.gdk.POINTER_MOTION_MASK)
		applet.connect("motion_notify_event", self._mouse_moved)
		applet.add(gtk.DrawingArea())
		applet.get_child().set_size_request(self.tank.get_width(), \
				self.tank.get_height())
		applet.get_child().connect("expose_event", lambda *foo: self.redraw())
		menu_xml = """<popup name='button3'>
		<menuitem name='self.spawn_fish' 
			verb='self.spawn_fish' _label='_Spawn new fish' />
		<menuitem name='self.show_config_dialog' 
			verb='self.show_config_dialog' _label='_Preferences...'
			pixtype='stock' pixname='gnome-stock-configure' />
		<menuitem name='show_about_box' 
			verb='show_about_box' _label='_About...'
			pixtype='stock' pixname='gnome-stock-about' />
		</popup>
		"""
		show_abt =  lambda *foo: gnome.ui.About( \
				"Blogfish", "1.0", "(c) 2004 David Hugh-Jones", \
				"Natural selection for your memes.", \
				["David Hugh-Jones (coding)", \
				"Anders Thorssell (underwater photography)", \
				"Ryan Morin (fish images)", \
				"Michael Waniek (fish images)"], \
				[]).run()
		verbs = [('self.show_config_dialog', \
				lambda *foo: self.show_config_dialog()), \
				('self.spawn_fish', lambda *foo: self.spawn_fish()), \
				('show_about_box', show_abt)]
		self.applet.setup_menu(menu_xml, verbs, None)

		for sig in (signal.SIGINT, signal.SIGTERM, \
				signal.SIGQUIT, signal.SIGHUP):
			signal.signal(sig, self._handle_signal)
	
	
	def run(self):
		FishTPServer.create_fishtp_server(self) # creates and runs new thread
		self.spawn_fish()
		self.turn_timeout = gtk.timeout_add(int(250 / self.speed), self.turn)
		self.peer_manager.run()
		if self.poss_host: self.peer_manager.try_peer(self.poss_host)
		self.applet.show_all()

	def turn(self):
		if random.random() < self.spawningRate: self.spawn_fish()
		if random.random() < 0.01 and self.fishes:
			t = random.choice(self.fishes).get_text()
			if t: self.add_text(t)
			
		# overcrowding
		if len(self.fishes) > self.maxFish:
			debug("Overcrowding!")
			self.fishes.sort(lambda f1, f2: cmp(f1.fitness, f2.fitness))
			self.fishes[0].fitness = 0
		
		for fish in self.fishes:
			# aging 
			if random.random() < self.deathRate: 
				fish.fitness = fish.fitness - 1 
				
			# movement
			if random.random() < 0.005: 
				fish.facingleft = not fish.facingleft
			r = random.random()
			if r < 0.01:
				fish.climb = -3
			elif r > 0.99:
				fish.climb = 3
				
			if fish.facingleft:
				fish.x -= 1
			else:
				fish.x += 1
			if fish.climb < 0:
				fish.climb += 1
				if fish.y < self.tank.get_height() + 15: fish.y += 1
			if fish.climb > 0:
				fish.climb -= 1
				if fish.y > -15: fish.y -= 1

			# leaving
			if fish.x <= -15 or fish.x >= self.tank.get_width():
				if self.peer_manager.get_peers() and random.random() < 0.25:
					self.fishes.remove(fish)
					self.set_tooltip()
					self.peer_manager.send_fish_away(fish)
					continue
				else:
					if fish.x <= -15: fish.facingleft = 0
					else: fish.facingleft = 1
		
			# proximity detection
			if fish.facingleft:
				for fish2 in self.fishes:
					if fish2.x > fish.x and 14 <= fish2.x - fish.x <= 15 \
							and -15 < fish2.y - fish.y < 15: 
						self.interact(fish, fish2, self)

			# aging (actually only for scaling purposes...)
			if (fish.age < 20): fish.age += 1
			
			# death
			if (fish.fitness < 1):
				debug(fish.name + " died")
				self.fishes.remove(fish)
				self.set_tooltip()
				continue
			
		# oblivion
		for text in self.texts:
			text["countdown"] = text["countdown"] - 1
			text["x"] = text["x"] + text["xdir"]
			text["y"] = text["y"] + text["ydir"]
			text["bright"] = text["bright"] - 1
			if text["countdown"] == 0: self.texts.remove(text)
			
		self.redraw()
		return gtk.TRUE # for GTK
	
	def _fight(self, fish1, fish2):
		total = fish1.fitness + fish2.fitness
		if total < 2: return
		p = random.randint(1, total)
		if p > fish1.fitness:
			fish1.fitness = fish1.fitness - 1 
		else:
			fish2.fitness = fish2.fitness - 1
		
	def _shag(self, fish1, fish2, ocean):
		total = fish1.fitness + fish2.fitness
		if total < 2: return
		l = lambda: random.randint(1, total)
		if l() > fish1.fitness:	pb = fish2.pixbuf.copy()
		else: pb = fish1.pixbuf.copy()
		if l() > fish1.fitness: lucky = fish2
		else: lucky = fish1
		print("A new fish is born!")
		baby = Fish(lucky.name, pb, lucky.email, lucky.URL, lucky.blogURL, \
				lucky.words, 5, None, 1)
		if fish1.sex == "F": mother = fish1
		else: mother = fish2
		ocean.add_fish(baby, mother.x, mother.y)
		
	def interact(self, fish1, fish2, ocean):
		# nature red in tooth and claw
		r = random.random()
		if r < self.aggression and fish1.sex == fish2.sex: 
			self._fight(fish1, fish2)
		elif r < self.sexuality and fish1.sex != fish2.sex:
			self._shag(fish1, fish2, ocean)
		
	def redraw(self):
		tank_h = self.tank.get_height()
		tank_w = self.tank.get_width()
		tcolor = gtk.gdk.color_parse("#8cc")
		if not  self.applet.get_child().window: debug("NO WINDOW!!!")
		(w,h) = self.applet.get_child().window.get_size()
		win = gtk.gdk.Pixmap(self.applet.get_child().window, w, h) 
		pl = self.applet.get_child().create_pango_layout("")
		pl.set_font_description(pango.FontDescription("serif italibc light 10"))
		gc = win.new_gc()
		
		self.tank.render_to_drawable_alpha(win, \
				0,0,0,0,-1,-1,0,0,gtk.gdk.RGB_DITHER_NONE,0,0)	
		
		for text in self.texts:
			b = text["bright"]
			color = gtk.gdk.Color(1500 * b, 2000 * b, 2000 *b)
			pl.set_markup(text["words"])
			win.draw_layout(gc, text["x"],  text["y"], \
					pl, color, None)
		
		for fish in self.fishes:
			fpb = fish.get_gdk_pixbuf()
			if fish.age < 20:
				w = int(fpb.get_width() * fish.age / 20) + 1
				h = int(fpb.get_height() * fish.age / 20) + 1
				fpb = fpb.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
			if not fpb: continue # not sure why this happens
			area_w = min(fpb.get_width(), tank_w - fish.x)
			area_h = min(fpb.get_height(), tank_h - fish.y)
			if area_w > 0 and area_h > 0: 
				fpb.render_to_drawable_alpha(win, \
						0, 0, fish.x, fish.y, area_w, area_h, 0, 0, \
						gtk.gdk.RGB_DITHER_NONE, 0, 0)
	
		self.applet.get_child().window.draw_drawable(gc, win, \
				0, 0, 0, 0, -1, -1)
			
	def add_text(self, words):
		self.texts.append({'countdown':25, 'words':words, \
				'x': random.randint(0, self.tank.get_width()),
				'y': random.randint(0, self.tank.get_height()),
				'xdir': random.choice((-1, 1)),
				'ydir': random.choice((-1, 0, 1)),
				'bright': 32})
	
	def config_changed(self, client, cnxn_id, entry, xxx):
		k = re.sub("/apps/blogfish/", "", entry.key)
		v = entry.value
		self.__dict__[k] = get_config_value(k, v)
			
		if k == "backgroundFile":
			self.tank = gtk.gdk.pixbuf_new_from_file(v.to_string())
			# necessary?
			self.applet.get_child().set_size_request(self.tank.get_width(), \
				self.tank.get_height())
			self.applet.check_resize()
			
		if k == "speed":
			if self.turn_timeout: gtk.timeout_remove(self.turn_timeout)
			self.turn_timeout = gtk.timeout_add(int(250 / self.speed), \
					self.turn)
		
	def add_fish(self, fish, x = None,y = None):
		fish.facingleft = (random.random() < 0.5)
		if y is None: y = random.randint(0,20)
		if x is None: x = random.randint(0, self.tank.get_width())
		fish.y = y
		fish.x = x
		self.fishes.append(fish)
		self.set_tooltip()
	
	def spawn_fish(self, url = None, blogurl = None):
		if not url: url = self.URL
		if not blogurl: blogurl = self.blogURL
		debug("spawning fish with url " + url + " blogurl " + blogurl)
		try:
			pixbuf = gtk.gdk.pixbuf_new_from_file(self.fishFile)
		except:
			debug("Couldn't load image from file " + self.fishFile)
			return
		fish = Fish(self.fishName, pixbuf, self.email, url, \
				blogurl, self.fishWords, 5, None, 1)
		self.add_fish(fish)

	def set_tooltip(self):
		names = {}
		if self.fishes:
			tip = "Current fish:\n"
			for f in self.fishes:
				if f.name in names: names[f.name] += 1
				else: names[f.name] = 1
			for name, num in names.iteritems():
				# luckily fish name plurals are the same as the singular ;-)
				tip += "\n%s %s" % (num, name)
			fish = random.choice(self.fishes)
			text = fish.get_text() or ""
			if text:
				text = textwrap.fill(text, 40, initial_indent='  ')
				tip = tip + "\n\n%s says:\n" % fish.name 
				tip = tip + text 
		else:
			tip = "No fish in ocean"
		
		self.tooltip.set_tip(self.applet, tip)
		
	
	def show_config_dialog(self):
		if not self.config_dialog:
			self.config_dialog = OceanConfig(self)
		self.config_dialog.run()
	
	def _drag_data_received(self, widget, context, x, y, sel, ttype, time):
		debug("got drag signal: " + sel.data)
		text = sel.data
		if text.endswith("xml"): # not subtle
			self.spawn_fish(None, text)
		else:
			self.spawn_fish(text, None)
		
	
	def _button_pressed(self, applet, event):
		# event is a gtk.gdk.Event of type BUTTON_PRESS
		if event.button == 3:
			pass
		#	self.contextMenu.popup(None, None, None, event.button, event.get_time())
		elif event.button == 1:
			x = event.x
			y = event.y
			self.fishes.sort( \
					lambda f1, f2: cmp(f1.x - x + f1.y - y, \
					f2.x - x + f2.y - y))
			for fish in self.fishes:
				if x - fish.x < 25 and y - fish.y < 25:
					fish.show_properties_dialog(applet)
					return
	
	def _mouse_moved(self, applet, event):
		x = event.x
		y = event.y
		for fish in self.fishes:
			if x -fish.x < 25 and y - fish.y < 25:
				applet.window.set_cursor(self.fish_cursor)		
				return
		applet.window.set_cursor(None)
	
	def _handle_signal(self, signum, frame):
		debug("handling signal")
		self.applet.destroy()
	
	def quit(self):
		# called when applet is destroyed
		# applet also calls gtk.main_quit()
		self.fishes.sort(lambda f1, f2: cmp(f2.fitness, f1.fitness))
		for fish in self.fishes:
			# if they die, they die
			self.peer_manager.send_fish_away(fish)
		self.peer_manager.shutdown()

