#!/usr/bin/python

import re
import gtk
import gconf
import gobject
import gtk.gdk
import gnome.ui
import gtk.glade
from blogfish.blogfish_functions import debug, get_config_value, \
		set_config_value

class OceanConfig:

	def __init__(self, ocean):
		self.ocean = ocean
		self.client = gconf.client_get_default()
		self.dialog = None
		self.client.add_dir("/apps/blogfish", gconf.CLIENT_PRELOAD_RECURSIVE)

	def run(self):
		if not self.dialog: self.dialog = self.setup()
		self.dialog.show_all()
		gtk.main()

	def setup(self):
		glade_dir = get_config_value("gladeDir")
		xml = gtk.glade.XML(glade_dir + "/config_dialog.glade")
		client = self.client

		w = {} # widgets
		entries = ( 'fishName', 'email', 'URL', 'blogURL', \
				 'fishWords',  'port', 'peerListFrom')
		spins = ('maxFish', 'maxPeers')
		pixmaps = ('backgroundFile', 'fishFile')
		checkbuttons = ('passiveServer',)
		scales = ('speed', 'aggression', 'sexuality', 'deathRate', \
				'spawningRate')
		
		widget_names = entries + spins + pixmaps + checkbuttons + scales
		
		for key in entries:
			w[key] = xml.get_widget(key + 'Entry')
			if not w[key]: print("no %s" % key)
			w[key].set_text(str(get_config_value(key))) # str because of "port"
		for key in spins:
			w[key] = xml.get_widget(key + 'SpinButton')
			w[key].set_value(get_config_value(key))
			w[key].connect("value-changed", self._set_config)
		for key in pixmaps:
			w[key] = xml.get_widget(key + 'PixmapEntry')
			w[key].set_filename(get_config_value(key))
			w[key].connect("activate", self._set_config)
			w[key].connect("changed", self._set_config)
		for key in checkbuttons:
			w[key] = xml.get_widget(key + 'CheckButton')
			w[key].set_active(get_config_value(key))
			w[key].connect("toggled", self._set_config)
		for key in scales:
			w[key] = xml.get_widget(key + 'Scale')
			w[key].set_value(get_config_value(key))
			w[key].connect("value-changed", self._set_config)

		for key in spins + entries:
			w[key].connect("focus_out_event", self._set_config)
			w[key].connect("activate", self._set_config)
			
		for key in widget_names:
			w[key].set_data('key', key)

		pl = xml.get_widget('portLabel')
		l = lambda entry, *args: \
				w['port'].set_sensitive(not entry.get_active()) \
				or pl.set_sensitive(not entry.get_active())
		w['passiveServer'].connect('toggled', l)
		l(w['passiveServer'])

		addButton = xml.get_widget('addPeersButton')
		addEntry = xml.get_widget('addPeersEntry')
		removeButton = xml.get_widget('removePeersButton')
		tv = xml.get_widget('peersTreeView')
		tv.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
		if self.ocean:
			# BAD PRAWN. Mix GUI with guts.
			tm = self.ocean.peer_manager.get_list_store()
			tv.set_model(tm)
			cell = gtk.CellRendererText()
			tvc = gtk.TreeViewColumn("Peer address", cell, markup=0)
			tv.append_column(tvc)
			tvc.add_attribute(cell, "text", 0)
			removeButton.connect('clicked', self._remove_peers, tv)
			addButton.connect('clicked', self._add_peer, addEntry)
			addEntry.connect('activate', self._add_peer, addEntry)
		
		dg = xml.get_widget("configDialog")
		dg.connect("delete_event", lambda *args: dg.hide_all() or 1)
		dg.connect("response", lambda *args: dg.hide_all() or 1)
		xml.get_widget("closeButton").connect("clicked", \
				lambda *args: dg.response(gtk.RESPONSE_CLOSE))

		return dg
		
	def _set_config(self, entry, *args):
		if isinstance(entry, gtk.CheckButton):
			val = entry.get_active()
		elif isinstance(entry, gnome.ui.PixmapEntry):
			val = entry.get_filename()
		elif isinstance(entry, gtk.Range):
			val = entry.get_value()
		else:
			val = entry.get_text()
		key = entry.get_data('key')
		set_config_value(key, val)
		
	def _add_peer(self, entry, *args):
		addEntry = args[-1]
		if self.ocean:
			self.ocean.peer_manager.try_peer(addEntry.get_text())
		addEntry.set_text("")

	def _remove_peers(self, entry, *args):
		tv = args[-1]
		model, paths = tv.get_selection().get_selected_rows()
		for path in paths:
			self._remove_peer(model, path, model.get_iter(path))

	def _remove_peer(self, treemodel, path, iter):
		peer = treemodel.get_value(iter, 0)
		peer = re.sub(r"</?span.*?>", "", peer)
		if self.ocean:
			self.ocean.peer_manager.remove_peer(peer)
	
#	def fill_out_peers_list(self, ls, tv):
#		ls.clear()
#		if self.ocean is not None:
#			peers = self.ocean.peer_manager.get_peers()
#			asked_peers = self.ocean.peer_manager.get_asked_peers()
#		else:
#			peers = ['some', 'test', 'peers']
#			asked_peers = ['asked for peer']
#
#		tvc = tv.get_column(0)
#		cell = tvc.get_cell_renderers()[0]
#		for peer in peers:
#			ls.append([peer])
#		for peer in asked_peers:
#			ls.append(["<span foreground='grey'>" + peer + "</span>"])
#		tv.show()
		
	
		
	def peer_dialog_response(self, dialog, id, treeView):
		ls = treeView.get_model().get_model()
		if id == 1: # add
			def add_peer(wid, id, entry):
				if id == gtk.RESPONSE_OK:
					t = entry.get_text()
					t = t.replace("http://","") 
					debug("in add_peer")
					self.ocean.peer_manager.try_peer(t)

			dg = gtk.Dialog("Add peer", dialog, 0, \
					(gtk.STOCK_ADD, gtk.RESPONSE_OK))
			entry = gtk.Entry()
			entry.connect("activate", lambda *foo: dg.response(gtk.RESPONSE_OK))
			dg.vbox.add(gtk.Label("Peer URL (e.g. www.fish.net:1114)"))
			dg.vbox.add(entry)
			dg.vbox.set_spacing(5)
			dg.set_border_width(10)
			dg.set_has_separator(0)
			dg.connect("response", add_peer, entry)
			dg.show_all()
			dg.run()
			dg.destroy()
			
		else: # remove
			sel = treeView.get_selection()
			pm = self.ocean.peer_manager
			sel.selected_foreach(lambda tm, path, iter: \
					pm.remove_peer(tm.get_value(iter, 0)))
		
if __name__ == "__main__":
	oc = OceanConfig(None)
	oc.run()
